﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace Ranges
{
    public class IntegerRange : Range<int>
    {
        public IntegerRange(int start , int end ) : base( start , end , 1 )
        {

        }
    }

    public class DayRange : Range<DateTime>
    {
        public DayRange(DateTime start, DateTime end )
            : base(start.Date, end.Date, d => d.AddDays(1), d => d >= start && d <= end )
        {

        }
    }

    public enum Places { First, Second, Third, Fourth };

    public class PlacesRange : Range<Places>
    {
        public PlacesRange(Places start, Places end)
            : base(start, end, p => p + 1, p => p>= start && p <= end)
        {

        }
    }
        
    class Program
    {
        static void Main(string[] args)
        {
            IntegerRange range = new IntegerRange(1, 10);

            PrintRange(range);
            Console.WriteLine();

            (range.Except(new Range<int>(8, 20,1)))
                .ToList()
                .ForEach(i => Console.WriteLine(i));

            range.Except( new int[] { 2,4,6,8 } ).ToList().ForEach( i=>Console.WriteLine(i) );
          
           PrintRange( new Range<double>(1,10,0.25));

           PrintRange( new PlacesRange(Places.First , Places.Fourth ) );

           PrintRange( new Range<int>(10,1,-1,true) );

           Console.WriteLine(100.In(new IntegerRange(0, 20)));
           Console.WriteLine(15.In(new IntegerRange(0, 20)));

           DayRange days = new DayRange(new DateTime(2009, 1, 1), new DateTime(2009, 12, 30));

           if (DateTime.Now.In(days))
           {
               Console.WriteLine("You are in Year 2009");
           }
          
        }

        private static void PrintRange<T>(Range<T> range) 
        {
            Console.WriteLine();
            Console.WriteLine("Range : {0} " , range);
            foreach (T item in range)
            {
                Console.WriteLine(item);
            }

        }
    }
}
