﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace Ranges
{
    public class Range<T> : IEnumerable<T>
    {
        public T Start { get; private set; }
        public T End { get; private set; }

        private Func<T, T> next;
        private Func<T, bool> isIn;

        public Range(T start, T end, T step)
            : this(start, end, step, false)
        {

        }

        public Range(T start, T end, T step, bool highToLow)
        {
            InitStartAndEnd(start, end);

            ParameterExpression current = Expression.Parameter(typeof(T), "current");

            Expression<Func<T, T>> nextExpr = Expression.Lambda<Func<T, T>>(
                Expression.Add(
                 current,
                 Expression.Constant(step)),
                 current);

            Expression<Func<T, bool>> isInExpr = null;

            if (highToLow)
            {
                isInExpr = Expression.Lambda<Func<T, bool>>(
                    Expression.And(
                        Expression.GreaterThanOrEqual(current, Expression.Constant(end)),
                        Expression.LessThanOrEqual(current, Expression.Constant(start))),
                        current);
            }
            else
            {
                isInExpr = Expression.Lambda<Func<T, bool>>(
                    Expression.And(
                        Expression.GreaterThanOrEqual(current, Expression.Constant(start)),
                        Expression.LessThanOrEqual(current, Expression.Constant(end))),
                        current);
            }


            isIn = isInExpr.Compile();
            next = nextExpr.Compile();
        }

        public Range(T start, T end, Func<T, T> next, Func<T, bool> isIn)
        {
            InitStartAndEnd(start, end);
            this.next = next;
            this.isIn = isIn;
        }

        public bool IsIn(T val)
        {
            return isIn(val);
        }

        public override string ToString()
        {
            return String.Format("{0}..{1}", Start, End);
        }

        private void InitStartAndEnd(T start, T end)
        {
            Start = start;
            End = end;
        }

        public IEnumerator<T> GetEnumerator()
        {
            T current = Start;
            yield return current;

            while (isIn(current))
            {
                current = next(current);
                yield return current;
            }
        }


        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }


    }
}
